import json
import os
import sys

from pycollimator import Api, global_variables
import pytest
from tests import config


def list_includes(lst, predicate):
    for item in lst:
        if predicate(item):
            return True
    return False


def test_projects_list():
    response = Api.get_projects()
    assert isinstance(response, object)

    projects = response["projects"]
    assert len(projects) > 0
    assert len(projects) == response["count"]

    project_uuid = global_variables.GlobalVariables.project_uuid()
    found = False
    for project in projects:
        assert isinstance(project, dict)
        if project["uuid"] == project_uuid:
            found = True
            break
    assert found


def test_models_list():
    response = Api.get_project()
    assert isinstance(response, dict)

    models = response["models"]
    assert len(models) > 0

    # check that we find all the expected models in this project
    expected_models = config["models"]
    assert len(expected_models) > 0
    fixtures_root = os.environ.get("FIXTURES_ROOT", "")

    for name, _modelcfg in expected_models.items():
        with open(f"{fixtures_root}tests/fixtures/{name}.json", "r") as f:
            model_data = json.loads(f.read())
        assert list_includes(models, lambda item: item["name"] == model_data["name"])


if __name__ == "__main__":
    sys.exit(pytest.main([__file__]))



import uuid

PATH_DELIMITER = "."


def is_uuid(s: str) -> bool:
    try:
        uuid.UUID(str(s), version=4)
        return True
    except ValueError:
        return False


# technically a nested path.
def is_path(s: str) -> bool:
    return s and PATH_DELIMITER in s



import json
import os
import pathlib
from functools import lru_cache

import pycollimator as collimator


config = None
ENV = None


@lru_cache(maxsize=None)
def setup_auth_token():
    global config
    global ENV

    if ENV is None:
        ENV = os.environ.get("ENV", "dev")
        print("ENV is", ENV)

    if ENV == "local":
        token = None
    elif ENV == "dev":
        path = pathlib.Path(__file__).parent.resolve().parent.resolve()
        token_file_path = os.path.join(path, "secrets", "token.txt")
        with open(token_file_path, "r") as f:
            token = f.read().strip()
    else:
        raise NotImplementedError(f"unspported ENV: {ENV} (should be either local or dev)")

    config_file_path = os.path.join(pathlib.Path(__file__).parent.resolve(), f"config-{ENV}.json")
    with open(config_file_path, "r") as f:
        config = json.loads(f.read())
        api_url = config["api_url"]
    collimator.global_variables.set_auth_token(token=token, api_url=api_url)

    projects = collimator.Api.get_projects()["projects"]
    project_uuid = None
    for p in projects:
        if p["title"] == config["project_title"]:
            project_uuid = p["uuid"]
            break

    if project_uuid is None:
        raise ValueError(f"project with title '{config['project_title']} 'not found")

    collimator.global_variables.set_auth_token(token=token, project_uuid=project_uuid, api_url=api_url)
    return token


@lru_cache(maxsize=None)
def get_model_uuid(name: str) -> str:
    global config
    return config["models"][name]["uuid"]


def get_model_name(short_name: str) -> str:
    fixtures_root = os.environ.get("FIXTURES_ROOT", "")
    with open(f"{fixtures_root}tests/fixtures/{short_name}.json", "r") as f:
        model_data = json.loads(f.read())
        return model_data["name"]


def __setup():
    os.environ["CI"] = "true"
    # collimator.Log.set_level("TRACE")
    setup_auth_token()


__setup()



import typing as T
import pandas as pd
import tempfile
import os


from pycollimator.api import Api
from pycollimator.error import CollimatorUnexpectedError
from pycollimator.hash import Hash
from pycollimator.log import Log


class SimulationHashedFile:
    def __init__(self, data: T.Any) -> None:
        if not isinstance(data, pd.DataFrame):
            raise ValueError("Unsupported data type. Must be: pandas.DataFrame")
        self._hash = None
        self.file = None
        self.data = data
        self.data.index.name = "time"
        self.content_type = "text/csv"

    @property
    def hash(self) -> str:
        self._ensure_write()
        if self._hash is None:
            self._hash = Hash.sha256sum(self.file.name)
        return self._hash

    def upload(self, model_uuid: str, simulation_uuid: str) -> None:
        self._ensure_write()
        # Uses hashed file name to indicate EFS file to pull data from.
        return self._do_upload(model_uuid, simulation_uuid, self.hash, self.file.name)

    def _ensure_write(self):
        if self.file is not None:
            return
        self.file = tempfile.NamedTemporaryFile(prefix=".simdata-", suffix=".csv", mode="w+", dir=".", delete=False)
        # Close the file because Windows can't have two handles open at once.
        # FIXME: we should clean up the file after upload
        self.file.close()
        Log.debug(f"Writing csv data to {self.file.name}")
        self.data.to_csv(self.file.name, index=True, header=True)

    def _do_upload(self, model_uuid: str, simulation_uuid: str, hash_value: str, path: T.Any) -> None:
        Log.debug(f"Creating simulation hashed file {path} with hash {hash_value}")
        content_length = os.path.getsize(path)
        body = {
            "hash_type": "sha256",
            "hash_value": hash_value,
            "content_type": "text/csv",
            "content_length": content_length,
        }
        response = Api.simulation_hashed_file_create(model_uuid, simulation_uuid, body)
        if response.get("in_cache", False) is True:
            Log.debug("Simulation hashed file is already in cache")
        elif response.get("upload_url", None) is not None:
            Log.debug("Uploading simulation hashed file to url:", response["upload_url"])
            response = Api.simulation_hashed_file_upload(path, response["upload_url"])
        else:
            raise CollimatorUnexpectedError("file is not cached but no upload url was returned")
        return response



from absl import app
from absl import flags
import pycollimator as pycol

FLAGS = flags.FLAGS

flags.DEFINE_string("token", None, "Client token.")
flags.DEFINE_string("project_uuid", None, "Project UUID.")
flags.DEFINE_string("api_url", "https://dev.collimator.ai", "Project name.")
flags.DEFINE_string("file", None, "File to upload.")

# Required flag.
flags.mark_flag_as_required("token")
flags.mark_flag_as_required("project_uuid")
flags.mark_flag_as_required("file")


def main(argv):
    del argv  # Unused.

    pycol.global_variables.set_auth_token(FLAGS.token, api_url=FLAGS.api_url, project_uuid=FLAGS.project_uuid)

    pycol.Api.upload_file(FLAGS.file, overwrite=True)


if __name__ == "__main__":
    app.run(main)



import sys

from pycollimator import Api, utils
import pytest
from tests import setup_auth_token, ENV


def test_auth_token():
    if ENV == "local":
        return

    token = setup_auth_token()

    print("toekn is", token)
    assert utils.is_uuid(token)


def test_is_uuid():
    assert utils.is_uuid("0f8c8d8f-a0b2-4b1c-b8d9-d5f5c9d5f5c9")
    assert utils.is_uuid("0f8c8d8f-a0b2-4b1c-b8d9-d5f5c9d5f5c9-123") is False
    assert utils.is_uuid("0f8c8d8f-a0b2-4b1c-b8d9-d5f5c9d5f5cZ") is False
    assert utils.is_uuid("1234") is False
    assert utils.is_uuid(None) is False


def test_api_status():
    response = Api.get(f"{Api.BASE_PATH}/status")
    assert response["status"] == "OK"


if __name__ == "__main__":
    sys.exit(pytest.main([__file__]))



import io
import control

import numpy as np
import pandas as pd

from pycollimator.error import NotFoundError
from pycollimator.i18n import N
from pycollimator.models import Model
from pycollimator.utils import is_path
from pycollimator.diagrams import Block


class SimulationResults:
    def __init__(self, stream, model: Model):
        if isinstance(stream, str):
            stream = io.StringIO(stream)
        self._raw_df = pd.read_csv(stream)
        self._model = model

    def __repr__(self) -> str:
        return (
            f"<{self.__class__.__name__} {len(self.to_pandas().index)} rows x {len(self.to_pandas().columns)} columns>"
        )

    def __getitem__(self, item):
        if item == "time":
            return self._raw_df["time"]  # pylint: disable=unsubscriptable-object
        if isinstance(item, Block):
            blk = item
        elif is_path(item):
            return self.to_pandas(path=item)
        else:
            # todo: do a search the model graph that constructs path 1
            try:
                blk = self._model.find_block(name=item, case=True)
            except NotFoundError:
                try:
                    blk = self._model.find_block(name=item, case=False)
                except NotFoundError:
                    blk = self._model.find_block(item)
        df = self.to_pandas(path=blk.path)
        return df

    def to_pandas(self, pattern=None, name=None, path=None, type=None, case=False, item=None) -> pd.DataFrame:
        # if nothing is specified, return all columns
        if pattern is None and name is None and path is None and type is None and item is None:
            cols = self._raw_df.columns[1:]  # remove time column and add back as index
        else:  # find specific block(s) to return results of
            # matches regardless of port specified or not, as well as all outputs of specified block
            if is_path(pattern):
                block_paths = [pattern]
            elif path:
                block_paths = [path]
            elif isinstance(item, Block):
                block_paths = [item.path]
            else:  # when given name or type, must query for blocks first.
                blocks = self._model.find_blocks(pattern=pattern, name=name, type=type, case=case)
                block_paths = []
                for block in blocks:
                    block_paths.append(block.path)
            cols = []
            for col in self._raw_df.columns:
                last_index = col.rfind(".")
                block_path = col[: col.rfind(".")]
                if (last_index != -1 and (block_path in block_paths)) or (col in block_paths):
                    cols.append(col)
        if len(cols) == 0:
            raise NotFoundError(
                N(
                    f"Simulation results '{self}' of model '{self._model}' has no results for "
                    f"(pattern='{pattern}' name='{name}' path='{path}' type='{type}' case='{case}' item='{item}') "
                )
            )

        df = self._raw_df[cols]  # pylint: disable=unsubscriptable-object
        df.columns = cols
        df.index = self._raw_df["time"]  # pylint: disable=unsubscriptable-object
        df.index.name = "time"
        return df

    @property
    def columns(self) -> list:
        return self.to_pandas().columns.to_list()


class LinearizationResult:
    def __init__(self, A, B, C, D):
        self.A = A
        self.B = B
        self.C = C
        self.D = D

    def to_state_space(self):
        """
        convert the linearization result to a python control object
        """
        return control.StateSpace(self.A, self.B, self.C, self.D)

    def __repr__(self):
        def tostring(x):
            return np.array2string(x, separator=",").replace("\n", "").replace(" ", "")

        return (
            f"<{self.__class__.__name__} A={tostring(self.A)} B={tostring(self.B)} "
            + f"C={tostring(self.C)} D={tostring(self.D)}>"
        )

    @classmethod
    def _from_csv(cls, csv_text):
        a_mat, b_mat, c_mat, d_mat = cls.__reshape_results_linearization(csv_text)
        return cls(a_mat, b_mat, c_mat, d_mat)

    # FIXME this format is not very robust
    @classmethod
    def __reshape_results_linearization(csv, results_text):
        read_st = -1
        lines = results_text.splitlines()
        for each in lines:
            if read_st == -1:
                if each[:3] == "dim":
                    # do the processing common for all matrices to get their dimensions
                    line_data = each.split(",")
                    mat_name = line_data[1]
                    tmp_dim = np.fromiter((float(x) for x in line_data[2:4]), dtype=int)
                    read_st = 0
                    if mat_name == "A":
                        a_dims = tmp_dim.copy()
                    elif mat_name == "B":
                        b_dims = tmp_dim.copy()
                    elif mat_name == "C":
                        c_dims = tmp_dim.copy()
                    elif mat_name == "D":
                        d_dims = tmp_dim.copy()
                    else:
                        print("unrecognized matrix name")
            else:
                # do the processing common for all matrices tobget their data
                line_data = each.split(",")
                line_data = line_data[:-1]
                # print(line_data)
                tmp = np.fromiter((float(x) for x in line_data), dtype=float)
                read_st = -1
                if mat_name == "A":
                    a_mat = np.reshape(tmp, a_dims)
                elif mat_name == "B":
                    b_mat = np.reshape(tmp, b_dims)
                elif mat_name == "C":
                    c_mat = np.reshape(tmp, c_dims)
                elif mat_name == "D":
                    d_mat = np.reshape(tmp, d_dims)
                else:
                    print("unrecognized matrix name")
        # get column names instead of uuids.
        return a_mat, b_mat, c_mat, d_mat



import os
import unittest

from pycollimator import Api
from pycollimator.error import CollimatorApiError
from tests import setup_auth_token, ENV


class TestUploadFile(unittest.TestCase):
    def setUp(self) -> None:
        self.token = setup_auth_token()
        self.fixtures_root = os.environ.get("FIXTURES_ROOT", "")
        return super().setUp()

    def test_upload_file(self):
        response = Api.upload_file(f"{self.fixtures_root}tests/fixtures/stop.png", overwrite=True)

        self.assertEqual("image/png", response["summary"]["content_type"])
        self.assertEqual("stop.png", response["summary"]["name"])
        self.assertEqual(356793, response["summary"]["size_in_bytes"])
        self.assertEqual("upload_completed", response["summary"]["status"])

    def test_upload_file_fails(self):
        if ENV == "local":
            return

        with self.assertRaises(CollimatorApiError) as context:
            Api.upload_file(f"{self.fixtures_root}tests/fixtures/stop.png", overwrite=False)

        self.assertTrue("File with this name already exists: stop.png" in str(context.exception))


if __name__ == "__main__":
    unittest.main()



import hashlib
from pycollimator.log import Log


class Hash:
    @classmethod
    def sha256sum(cls, filename: str) -> str:
        sha256_hash = hashlib.sha256()
        with open(filename, "rb") as f:
            for byte_block in iter(lambda: f.read(4096), b""):
                sha256_hash.update(byte_block)
            value = sha256_hash.hexdigest()
            Log.trace(f"SHA256 hash of {filename} is {value}")
            return value
